@@ -1,5 +1,5 @@
 var _ = require('underscore'),
-    templates = require('./templates');
+  templates = require('./templates');
 
 module.exports = function(grunt, callback) {
   // Deferring to here in case we have a build for parser, etc as part of this grunt exec
@@ -8,12 +8,17 @@ module.exports = function(grunt, callback) {
   var templateSizes = {};
   _.each(templates, function(info, template) {
     var src = info.handlebars,
-        compiled = Handlebars.precompile(src, {}),
-        knownHelpers = Handlebars.precompile(src, {knownHelpersOnly: true, knownHelpers: info.helpers});
+      compiled = Handlebars.precompile(src, {}),
+      knownHelpers = Handlebars.precompile(src, {
+        knownHelpersOnly: true,
+        knownHelpers: info.helpers
+      });
 
     templateSizes[template] = compiled.length;
     templateSizes['knownOnly_' + template] = knownHelpers.length;
   });
-  grunt.log.writeln('Precompiled sizes: ' + JSON.stringify(templateSizes, undefined, 2));
+  grunt.log.writeln(
+    'Precompiled sizes: ' + JSON.stringify(templateSizes, undefined, 2)
+  );
   callback([templateSizes]);
 };
