@@ -8,5 +8,6 @@ module.exports = {
     bar: true
   },
 
-  handlebars: '{{foo person "person" 1 true foo=bar foo="person" foo=1 foo=true}}'
+  handlebars:
+    '{{foo person "person" 1 true foo=bar foo="person" foo=1 foo=true}}'
 };
