@@ -1,13 +1,15 @@
 #!/bin/bash
 
-cd "$( dirname "$( readlink -f "$0" )" )" || exit 1
+set -e
+
+cd "$( dirname "$( readlink -f "$0" )" )"
 
 for i in */test.sh ; do
   (
     echo "----------------------------------------"
     echo "-- Running integration test: $i"
     echo "----------------------------------------"
-    cd "$( dirname "$i" )" || exit 1
-    ./test.sh || exit 1
+    cd "$( dirname "$i" )"
+    ./test.sh
   )
-done
\ No newline at end of file
+done
